function Irec = CellularAutomatonRecovery(SImg, Smask)
    % Cellular Automaton-based Recovery
    % input arguments:
    % SImg ----> Sampled Image
    % Smask ----> Sampling Mask
    % output argument:
    % Irec ----> recunstructed image
        
    [h, w] = size(SImg); % determining the size of image
    
    Mask0 = Smask;
    
    Wfilter = 3; % initial size of filtering window 
    sig = 1; % initial standard deviation
    alpha = 1.05; % a constant coefficient
    Irec = SImg; % initialization of reconstructed image
    Mask = Smask; % initialization of mask
    Ndc = length(find(Mask(:) == 0)); % the number of dead cells
  
    while Ndc ~= 0
        Wh = floor(Wfilter/2);  
        Weight = fspecial('gaussian', [Wfilter, Wfilter], sig);
        for r = 1:1:h
            for c = 1:1:w
                if Smask(r, c) == 0 % dead cell
                    flag = 0; % Flag initializes with false.
                    temp = 1;
                    for rw = -Wh:1:Wh
                        for cw = -Wh:1:Wh
                            ii = r + rw;
                            jj = c + cw;
                            
                            if ii < 1              
                                ii = 1;
                            elseif ii > h
                                ii = h;
                            end % end of if ii 
                            if jj < 1              
                                jj = 1;
                            elseif jj > w
                                jj = w;
                            end % end of if jj

                            if Smask(ii, jj) == 1 % collecting live cells
                                f(temp) = SImg(ii, jj);
                                f_weight(temp) = Weight(rw+ceil(Wfilter/2), cw+ceil(Wfilter/2));
                                
                                flag = 1; % Flag is now true.
                                temp = temp + 1;
                            end % end of if Smask
                        end % end of for cw
                    end % end of for rw
                    
                    if flag == 1 % to be true
                        if isnan(sum(f.*f_weight)/sum(f_weight)) == 0 % Condition to prevent NaN
                             Irec(r, c) = sum(f.*f_weight)/sum(f_weight);
                        else
                            Irec(r, c) = 0;
                        end               
                        Mask(r, c) = 1; % survive for next generation 
                    end % end of if flag
                    
                    clear f f_weight
                    
                end % end of if
            end % end of for c
        end % end of for r
        
        Wfilter = Wfilter + 2; % updating the size of window
        sig = alpha*sig;
        SImg = Irec; % updating the sampled image
        Smask = Mask; % updating the sampling mask
        Ndc = length(find(Mask(:) == 0)); % updating the number of dead cells
               
    end % end of while
    
    Irec = PostProcess(Irec, Mask0); % Post Processing after final iteration
    
end % end of function

