function [roundedf_roundedx, roundedf_roundedy] = Filtering(fgray, Myi, Mxi, h, w)
    % Filtering
    % input arguments: the gray-level image fgray, the filtering kernels 'Myi' and 'Mxi', and the heigth 'h' and width 'w' of the image   
    % output argumets: the derivatives of the image in x and y directions, i.e. roundedf_roundedx, roundedf_roundedy
    
    Myi = Myi(:)';
    Mxi = Mxi(:)';
    Wfilter = sqrt(length(Myi)); % size of square window
    Wh = floor(Wfilter/2);
    % ************************* Preallocation *****************************
    fsi = zeros(1, Wfilter*Wfilter);
    roundedf_roundedx = zeros(h, w);
    roundedf_roundedy = zeros(h, w);
    % *********************************************************************
    for r = 1:1:h
        for c = 1:1:w
            temp = 1;
            for rw = -Wh:1:Wh
                for cw = -Wh:1:Wh
                    ii = r + rw;
                    jj = c + cw;

                    if ii < 1              
                        ii = 1;
                    elseif ii > h
                        ii = h;
                    end % end of if ii 
                    if jj < 1              
                        jj = 1;
                    elseif jj > w
                        jj = w;
                    end % end of if jj
                    fsi(temp) = fgray(ii, jj);
                    temp = temp + 1;
                end
            end
            roundedf_roundedx(r, c) = sum(fsi.*Mxi);
            roundedf_roundedy(r, c) = sum(fsi.*Myi);
        end
    end
    % *********************************************************************
end

